from __future__ import absolute_import, print_function

# Copyright (c) 2010-2020 Andrew Dalke Scientific, AB (Sweden)
# All Rights Reserved.

## LICENSE ##

# This file contains proprietary and unpublished source code.
#
# You may use and modify this software ONLY under the terms of the
# License agreement you or your organization made with Andrew Dalke
# Scientific AB.
#
# If no other agreeement exists then you may use chemfp under the
# terms of the Chemfp Base License Agreement v1.0 or later.
# See http://chemfp.com/licenses/ for details.
#
# Unless required by applicable law or agreed to in writing, this
# software is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
# CONDITIONS OF ANY KIND, either express or implied.  See the License
# agreement for the specific language governing permissions and
# limitations under the License.

## /LICENSE ##

import os
import sys
import _chemfp
from _chemfp import (hex_isvalid, hex_popcount, hex_intersect_popcount,
                     hex_tanimoto, hex_tversky, hex_contains, hex_contains_bit,
                     hex_encode, hex_encode_as_bytes, hex_decode,
                     hex_from_bitlist, hex_to_bitlist,
                     hex_intersect, hex_union, hex_difference)
                    

from _chemfp import (byte_popcount, byte_intersect_popcount,
                     byte_tanimoto, byte_tversky, byte_contains, byte_contains_bit,
                     byte_hex_tanimoto, byte_hex_tversky,
                     CHEMFP_FLOAT_SCALE,
                     byte_from_bitlist, byte_to_bitlist,
                     byte_intersect, byte_union, byte_difference)


__all__ = ["byte_popcount", "byte_intersect_popcount",
           "byte_tanimoto", "byte_tversky", "byte_contains", "byte_contains_bit",
           "byte_to_bitlist", "byte_from_bitlist",
           "byte_intersect", "byte_union", "byte_difference",
           "hex_isvalid", "hex_popcount", "hex_intersect_popcount", 
           "hex_tanimoto", "hex_tversky", "hex_contains", "hex_contains_bit",
           "byte_hex_tanimoto", "byte_hex_tversky",
           "hex_to_bitlist", "hex_from_bitlist",
           "hex_intersect", "hex_union", "hex_difference",
           "hex_encode", "hex_encode_as_bytes", "hex_decode"
           # Don't export the popcount and alignment-related functions
               ]
    
#hex_decode = binascii.unhexlify

_sizes = None
def get_sizes():
    global _sizes
    if _sizes is None:
        _sizes = tuple(_chemfp.get_size_name(i) for i in range(_chemfp.get_num_size_implementations()))
    return _sizes

_methods = None
def get_methods():
    global _methods
    if _methods is None:
        _methods = tuple(_chemfp.get_method_name(i) for i in range(_chemfp.get_num_methods()))
    return _methods

def get_method_families():
    return [_chemfp.get_method_family_name(i) for i in range(_chemfp.get_num_method_families())]

def get_size_methods():
    from collections import OrderedDict
    size_methods = OrderedDict()
    for i in range(_chemfp.get_num_size_implementations()):
        size_methods[_chemfp.get_size_name(i)] = _chemfp.get_size_method_name(i)
    return size_methods

def _get_size_index(size_name):
    try:
        return get_sizes().index(size_name)
    except ValueError:
        pass
    raise ValueError("Unknown size_name: %r" % (size_name,))
    
def _get_method_name_index(method_name):
    try:
        return get_methods().index(method_name)
    except ValueError:
        pass
    raise ValueError("Unknown method_name: %r" % (method_name,))
    
def set_size_method(size_name, method_name):
    size_name_index = _get_size_index(size_name)
    method_name_index = _get_method_name_index(method_name)
    _chemfp.set_size_method(size_name_index, method_name_index)

def get_options():
    return [_chemfp.get_option_name(i) for i in range(_chemfp.get_num_options())]

def get_option(option):
    return _chemfp.get_option(option)

def set_option(option, value):
    _chemfp.set_option(option, value)

def print_report(out=sys.stdout):
    from . import SOFTWARE
    if isinstance(SOFTWARE, type(u"")):
        software = SOFTWARE
    else:
        software = unicode(SOFTWARE, "ascii")
    print(u"== Configuration report for", software, u"==", file=out)
    print(u"Number of available threads:", _chemfp.get_num_threads(), file=out)
    print(u"Available method families:", " ".join(get_method_families()), file=out)
    print(u"Available methods:", " ".join(get_methods()), file=out)
    print(u"Size methods:", file=out)
    for size_name, method_name in get_size_methods().items():
        print(u"  %s: %s" % (size_name, method_name), file=out)
    print(u"Option settings:", file=out)
    for option in get_options():
        print(u"  %s: %s" % (option, _chemfp.get_option(option)), file=out)

def use_environment_variables(environ=None, outfile=sys.stderr):
    method_name_to_index = dict((name, i) for i, name in enumerate(get_methods()))
    size_name_to_index = dict((name, i) for i, name in enumerate(get_sizes()))
    
    if environ is None:
        environ = os.environ

    known = set()
    
    known.add("CHEMFP_LICENSE")
    known.add("CHEMFP_LICENSE_MANAGER")
    known.add("CHEMFP_REPORT")
    known.add("CHEMFP_GZIP")
    known.add("CHEMFP_GZCAT")
    known.add("CHEMFP_USE_SYSTEM_GZIP")
    report = environ.get("CHEMFP_REPORT", "0") == "1"

    # CHEMFP_FAMILIES is a comma-separated list of popcount family names.
    # They are set in the given order, so "popcnt,avx2" sets all of the
    # popcnt implementations then sets all of the avx2 implementations.
    
    name = "CHEMFP_FAMILIES"
    value = environ.get(name, "")
    if value:
        known.add(name)
        if report:
            print("INFO: Setting method families using CHEMFP_FAMILIES=%r" % (value,),
                  file=outfile)
        family_names = value.split(",")

        expanded_family_names = []
        for family_name in family_names:
            if family_name == "all":
                expanded_family_names.extend(get_method_families())
            elif not family_name:
                pass
            else:
                expanded_family_names.append(family_name)

        family_names = expanded_family_names
        for family_name in family_names:
            try:
                _chemfp.use_method_family(family_name)
            except ValueError as err:
                print("WARNING: Cannot use method family %r: %s"
                      % (family_name, str(err)), file=outfile)
            else:
                if report:
                    print("INFO: Applied methods from family %s" % (family_name,),
                           file=outfile)

    # Low-level control over each possible size.
    # CHEMFP_SIZE128_18 changes the method for "size128-18"
    for size_name in get_sizes():
        name = "CHEMFP_" + size_name.upper().replace("-", "_")
        known.add(name)
        try:
            method_name = environ[name]
        except KeyError:
            pass
        else:
            size_index = size_name_to_index[size_name]
            try:
                method_index = method_name_to_index[method_name]
            except KeyError:
                print("WARNING: Unable to use $%s = %r: %s" % (
                    (name, method_name, "unsupported method")),
                    file=outfile)
            else:
                try:
                    _chemfp.set_size_method(size_index, method_index)
                except ValueError as err:
                    print("WARNING: Unable to use $%s = %r: %s" % (
                        (name, method_name, err)),
                        file=outfile)

    # using CHEMFP_REPORT enables CHEMFP_REPORT_POPCOUNT,
    # CHEMFP_REPORT_INTERSECT, and CHEMFP_REPORT_ALGORITHM
    if report:
        set_option("report-popcount", 1)
        set_option("report-intersect", 1)
        set_option("report-algorithm", 1)

    # Options can also be configured individually
    for option in get_options():
        name = "CHEMFP_" + option.upper().replace("-", "_")
        known.add(name)
        try:
            value = environ[name]
        except KeyError:
            pass
        else:
            try:
                int_value = int(value)
            except ValueError:
                print("WARNING: Unable to use $%s = %r: not an integer value" % (
                      (name, value)), file=outfile)
                continue
            
            try:
                set_option(option, int_value)
            except ValueError as err:
                print("WARNING: Unable to use $%s = %r: %s" % (
                      (name, value, err)), file=outfile)

    known.add("CHEMFP_PRINT_CONFIG")
    if (environ.get("CHEMFP_PRINT_CONFIG", "0") == "1" or report):
        print_report(outfile)


    for k in environ:
        if not k.startswith("CHEMFP_"):
            continue
        if k not in known:
            print("WARNING: Unknown chemfp environment variable %r" % (k,),
                  file=outfile)

if __name__ == "__main__":
    use_environment_variables()
    print_report()
